#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2023 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>Custom Fonts>Constructing Tools>Custom Font File Splitter...
#     initial version
#         Splits a custom font file into smaller child files and displays these as new gimp images
#     version 01
#         Options to select number of child files and save the files to the original folder (over-writing the original larger file)
#     version 02
#         adds a layer to the final child file to signify that this is the last file to be used and mark other child layers as not being the final layer

from gimpfu import *
#from array import array
#import math
#import random

#from gtk import gdk
#w = gdk.screen_width()
#h = gdk.screen_height()

# Some global colours we may need to define

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)

#####
##### TEMP VALUES

# Some global colours we may need to define

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)



def do_stuff(image, layer, num_slices, do_save):
    #### Main Filter Proc
	
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)	
	
    #####
    ##### TEMP VALUES
    apply_to_custom = True
	
	
		
    dummy = 0
    #active_layer_id = pdb.gimp_image_get_active_layer(image)
    active_layer_id = False
	
	
    loop_ptr = 1
    first_char_layer_name = "none"
    num_layers, layer_ids = pdb.gimp_image_get_layers(image)
    loop = num_layers - loop_ptr
    layer_names_list = image.layers
    original_active_layer = image.active_layer
		
    num_multi_layers,other_layers,num_char_layers = 0,0,0
    visi_loop, first_char_layer  = 0, -1
	
    filename = pdb.gimp_image_get_filename(image)
    #pdb.gimp_message("filename = " + str (filename))
    file_base_ptr = filename.rfind(".")
    file_base = filename[0:file_base_ptr]
    #pdb.gimp_message("filename = " + str (filename) + " : file_base = " + str(file_base) )
	
	
	
    num_slices = int(num_slices)
    layers_per_file = int(num_layers // num_slices) # determine how many characters in each child file
    if int(num_layers % num_slices)>0: layers_per_file+= 1
    if layers_per_file<2: layers_per_file=2 #  sanity check
	
    #pdb.gimp_message("(num_layers % int(num_slices)) =  " + str((num_layers % int(num_slices))) )
    # set flag for extra layers to go in the last child file
    last_file_has_more = False
    if (num_layers % num_slices) > 0: last_file_has_more = True
	
    #pdb.gimp_message("layers_per_file =  " + str(layers_per_file) )
	
    debug_text = "filename = " + str(filename)
    debug_text += "\n\nfile_base = " + str(file_base)
    debug_text += "\n\nnum_slices = " + str(num_slices)
    debug_text += "\n\nlayers_per_file = " + str(layers_per_file)
    debug_text += "\n\nlast_file_has_more = " + str(last_file_has_more)
	
    # check for custom font layers
    num_char_layers = 0
    while visi_loop < num_layers: 
	    # get info about layer we are working on
        layer_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
        layer_name = layer_name.decode('utf-8')
        # check to see if the current layer is a custom font character layer
        test_for_custom_layer = fn_test_for_custom_layer(image, layer_names_list[visi_loop], layer_name)
			
        if test_for_custom_layer:
            if active_layer_id == False: active_layer_id = layer_names_list[visi_loop]
            num_char_layers += 1
            #if num_char_layers == ((layers_per_file)+1): split_loop_start = visi_loop
            #if num_char_layers == ((layers_per_file)): split_loop_start = visi_loop
            if first_char_layer < 0: 
                first_char_layer = visi_loop
                first_char_layer_name = layer_name
                layer_names_list[visi_loop].set_offsets(0, 0)
				
            
        pdb.gimp_item_set_visible(layer_names_list[visi_loop],False) # turn off visibility
        visi_loop += 1
		
    debug_text += "\n\nnum_char_layers = " + str(num_char_layers)
 
		
    if num_char_layers>layers_per_file:
        temp_image_counter = 0
        keep_going = True
        loop, loop_counter = 0, 0
		
		
		
		
        character_count = layers_per_file
        loop = layers_per_file
        file_counter = 0
		
        while (loop<num_layers):
            new_image = pdb.gimp_image_new(image.width, image.height, 0) # add a new image
            char_loop = 0
            while (char_loop < layers_per_file) and (loop<num_layers):
	            # get info about layer we are working on
                layer_name = pdb.gimp_item_get_name(layer_names_list[loop])
                layer_name = layer_name.decode('utf-8')
                # check to see if the current layer is a custom font character layer
                #test_for_custom_layer = fn_test_for_custom_layer(image, layer_names_list[loop], layer_name)
		
                test_for_custom_layer = True
                if test_for_custom_layer:
                    temp_layer_id = pdb.gimp_image_get_layer_by_name(image, layer_name) # get layer
                    new_layer_id = pdb.gimp_layer_new_from_drawable(temp_layer_id, new_image) # copy it
                    pdb.gimp_image_insert_layer(new_image, new_layer_id, None, -1) # add layer
				
                    temp_layer_copy = new_layer_id.copy(False) # make a copy of the layer
                    new_image.add_layer(temp_layer_copy,0)
                    new_image.remove_layer(temp_layer_copy) # remove temporary layer
                    pdb.gimp_item_set_name(new_layer_id, layer_name)
                    pdb.gimp_image_lower_item_to_bottom(new_image, new_layer_id)
		
                char_loop+=1
                loop+=1
            if loop<num_layers and char_loop == layers_per_file:
                extra_layer = pdb.gimp_layer_new(new_image, image.width, image.height, 1, "Layer_more_to_follow", 100, 28)
                pdb.gimp_image_insert_layer(new_image, extra_layer, None, -1) # add layer
                pdb.gimp_item_set_visible(extra_layer, True)
                pdb.gimp_image_lower_item_to_bottom(new_image, extra_layer)
                
            file_counter+=1
				
			
            if loop==num_layers:
                final_layer = pdb.gimp_layer_new(new_image, image.width, image.height, 1, "Layer_eof", 100, 28)
                pdb.gimp_image_insert_layer(new_image, final_layer, None, -1) # add layer
                pdb.gimp_item_set_visible(final_layer, True)
                pdb.gimp_image_lower_item_to_bottom(new_image, final_layer)
			
            if do_save < 2: new_display = pdb.gimp_display_new(new_image)
			
            if do_save>0: 
                new_file_name = file_base + "_" + str(file_counter) + ".xcf"
                pdb.gimp_xcf_save(1, new_image, new_layer_id, new_file_name, new_file_name)
			
			
        if do_save>0: 
            new_file_name = file_base + ".xcf"
            pdb.gimp_xcf_save(1, image, new_layer_id, new_file_name, new_file_name)
			
        loop = character_count
		
        while loop<num_layers: # remove layers from parent file
            image.remove_layer(layer_names_list[loop])
            loop += 1
        extra_layer = pdb.gimp_layer_new(image, image.width, image.height, 1, "Layer_more_to_follow", 100, 28)
        pdb.gimp_image_insert_layer(image, extra_layer, None, -1) # add layer
        pdb.gimp_item_set_visible(extra_layer, True)
        pdb.gimp_image_lower_item_to_bottom(image, extra_layer)
			
        if do_save>0: 
            new_file_name = file_base + ".xcf"
            pdb.gimp_xcf_save(1, image, new_layer_id, new_file_name, new_file_name)

        
        if first_char_layer_name <> "none": # the character layer to make visible on completion
            first_char_layer_id = pdb.gimp_image_get_layer_by_name(image, first_char_layer_name)
            pdb.gimp_item_set_visible(first_char_layer_id, True)
            pdb.gimp_image_set_active_layer(image,first_char_layer_id)
        
			
        #pdb.gimp_message("There are " + str(num_char_layers) + " character layers." + " There will be " + str(number_of_images) + " images produced with " + str(layers_per_file) + " characters in each new image.")
        #if remaining_layers> 0: pdb.gimp_message( "There will be one new image containing " + str(remaining_layers) + " characters." )
			
    else:
        if num_char_layers == 0:
            message_text = "There appears to be no custom font layers associated with this image or the required character layers are missing from this file."
        else:
            
            message_text = "There are not enough character layers in this image file to split into groups of " + str(int(layers_per_file)) + "."
        old_handler = pdb.gimp_message_get_handler()	
        pdb.gimp_message_set_handler(MESSAGE_BOX)
        pdb.gimp_message(message_text)
        pdb.gimp_message_set_handler(old_handler)    
	
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
	
######################################################################################################################################

    #### Useful Functions
# -------------------------------------------------------------------------------------
# -------------------------------------------------------------------------------------

def fn_test_for_custom_layer(img,l_id,l_name):
    # checks that the current layer is compatible with custom fonts
    test = False
    if len(l_name) == 1:
        test = True
    elif len(l_name) == 5 or len(l_name) == 6:
        
        if ".png" in l_name:
            test = True
            if len(l_name) == 6:
                part_name_val = unicode(l_name[0:2])
                if part_name_val <= 65535: test = False
        
    if l_name == "space.png":
        test = True
    else:
    
        # select the contents of the layer
        pdb.gimp_image_select_item(img,2,l_id)
        # Test to see if the selection actually contains something
        sel_is_empty = pdb.gimp_selection_is_empty(img)
        # If there are no pixels to work with flag the layer as not viable to work on
        if sel_is_empty: test = False
        # Deselect
        pdb.gimp_selection_none(img)
		
    return test
	
######################################################################################################################################
	
def fn_selection_info(im):
    # returns selection info
    selection,x1,y1,x2,y2 = pdb.gimp_selection_bounds(im)
    s_width, s_height = x2 - x1, y2 - y1
	
    return selection, x1, x2, y1, y2, s_width, s_height


######################################################################################################################################
	
def fn_selection_amount(sel_value,r_ness,adjustment):
    # 
    value = (math.sqrt(sel_value)*r_ness)*adjustment
    return value


######################################################################################################################################
	
def proc_messagebox(message):
    old_handler = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2)
    pdb.gimp_message(str(message))
    pdb.gimp_message_set_handler(old_handler)
	

######################################################################################################################################
	
register(
    "python_fu_Custom_Font_File_Splitter_02",
    "Provides options for splitting custom font files into smaller files",
    "Provides options for splitting custom font files into smaller files",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2023",
    "<Image>/Filters/Custom Fonts/Utility Tools/Custom Font File Splitter v02...",
    "RGB*",
    [
    (PF_SPINNER, "Number_splits", "Number of split Custom Font files to create: ", 2, (2, 10, 1)),
    (PF_OPTION, "save_option", "Save options", 2, ("No save - just show images in Gimp", "Save to original folder and show images in Gimp", "Save to original folder - do not show child images")),
    ],
    [],
    do_stuff)

main()
