#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2023 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>Custom Fonts>Constructing Tools>Custom Font File Stitcher...

#         Initial version
#              Intended to compliment Custom Font Splitter which breaks custom font files into smaller chunks
#              Takes the filename for the current image and adds the character layers from any files with the same base name plus a numeric suffix eg "_1"
#         Version 01
#              Linux OS threw an error so this version takes a different approach to building file names.
#         Version 02
#              Adds an option to use a child file (ie a file ending with numbers) as the starting file for stitching
#              Default options will not allow user to  use a child file (ie a file ending with numbers) as the starting file for stitching
#         Version 02b
#              Fix for identifying layer names in non-English languages
#         Version 03
#              removes the need for eof and more-to-follow layers in custom font files



from gimpfu import *
import os
#from array import array
#import math
#import random

#from gtk import gdk
#w = gdk.screen_width()
#h = gdk.screen_height()

# Some global colours we may need to define

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)

#####
##### TEMP VALUES




def do_stuff(image, layer, do_stitch, do_numbered_names):
    #### Main Filter Proc
	
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)	
	
    # sanity check
    num_layers, layer_ids = pdb.gimp_image_get_layers(image)
    layer_names_list = image.layers
    loop = 0
    while loop<num_layers:
        layer_id = layer_names_list[loop]
        layername = pdb.gimp_item_get_name(layer_id)
        layername = layername.decode('utf-8')
        check = fn_test_for_custom_layer(image,layer_id,layername)# test for custom font characters in the file we are loading
	
        loop+=1
    check = pdb.gimp_image_get_layer_by_name(image, "Layer_more_to_follow")
	
        
	
    do_debug = False
		
    dummy = 0
    active_layer_id = pdb.gimp_image_get_active_layer(image)
	
    if do_stitch:
	
        parent_filename = pdb.gimp_image_get_filename(image)
        file_base_ptr = parent_filename.rfind(".")
        file_base = parent_filename[0:file_base_ptr]
        
        debug_info = "Parent image filepath = " + str(parent_filename)
        #pdb.gimp_message("file_base = " + str(file_base) )
		
        file_suffix = 1 # number given to child file
        #do not allow stitcher to work with numbered custom font file names by default since we assume that this is a mistake on the part of the user
        suffix_ptr = file_base.rfind("_")
        if suffix_ptr == -1: # if there is no underscore character in the filebase name do not allow use of numbered filenames
            last_character = False
            do_numbered_names = False
        else:
            last_character = file_base[suffix_ptr+1:len(file_base) ] # get characters after the underscore
            if last_character.isdigit(): # if there are numbers at the end of the filebase name do not set do_numbered_names to False
                last_character = int(last_character)
            #last_character = file_base[len(file_base)-1]
			
        valid = True
        #pdb.gimp_message("file_base = " + str(file_base) + "\n\nlast_character = " + str(last_character) )
        if str(last_character).isdigit(): valid = False
        # The user has made a decision to use a numbered custom font file name as a pare
        if do_numbered_names and str(last_character).isdigit(): 
            valid = True
            file_suffix = int(last_character)+1
            file_base_ptr = parent_filename.rfind("_")
            file_base = parent_filename[0:file_base_ptr]
	
        if str(parent_filename)<>"None" and valid:
        
            debug_info += "\n\n" + "Parent image base = " + file_base
            xcf_file_list = [file_base] # create list for child filenames using currently associated filename as the parent
            folder_base_ptr = parent_filename.rfind("\\")
            folder_info = parent_filename[0:folder_base_ptr]
            debug_info += "\n\n" + "Parent folder path = " + folder_info
            #pdb.gimp_message(str(folder_info) )
            child_num = 0  
				
				
				
			
            #debug_info += "\n\n" + test_file
			
			
            #debug_info += "\n\n test file handle = " + str(fontimg)
            
				
            eof_layer = True
            child_file_list = []
            while eof_layer:
                child_filename = file_base + "_" + str(file_suffix) + ".xcf"
                child_file_list.append(child_filename)
                font_name_pointer, file_dummy = -1, 0
                try:
			
                    fh = open(child_filename, "r")
                except IOError:
                    eof_layer=False 
                    debug_info += str(child_filename) + " could not be found.\n"
                else:
                    fontimg = pdb.gimp_xcf_load(file_dummy,child_filename,child_filename)
                    #pdb.gimp_message("fontimg = " + str(fontimg) ) ####
                    num_layers, layer_ids = pdb.gimp_image_get_layers(fontimg)
                    layer_names_list = fontimg.layers
				
				
                    load_loop = 0
                    fontimg_layer_id = layer_names_list[load_loop]
				

                    debug_info += "\n\n" + "Child file " + str(file_suffix) + " = " + child_filename
				
                
				
                
                    while load_loop<num_layers:
                        fontimg_layer_id = layer_names_list[load_loop]
                        fontimg_layername = pdb.gimp_item_get_name(fontimg_layer_id)
                        if "Layer_eof" == fontimg_layername: 
                            eof_layer = False
                        else:
                            if fontimg_layername<>"Layer_more_to_follow":
                                copy_layer = pdb.gimp_layer_new_from_drawable(fontimg_layer_id,image)
                                temp = pdb.gimp_image_insert_layer(image, copy_layer, None, -1)
                                pdb.gimp_item_set_visible(copy_layer, False)
                                pdb.gimp_layer_set_offsets(copy_layer,0,0)
                                pdb.gimp_image_lower_item_to_bottom(image, copy_layer)
                                copy_layer_name = pdb.gimp_item_get_name(copy_layer).decode('utf-8')
                                if ".png" in copy_layer_name:
                                    dummy = 0
                                    copy_layer_name_ptr = copy_layer_name.rfind(".png")
                                    copy_layer_name = copy_layer_name[copy_layer_name_ptr-1]+".png"
                                    pdb.gimp_item_set_name(copy_layer, copy_layer_name)
                        load_loop += 1
                    
                #loop += 1
				
                
                file_suffix += 1
                #if file_suffix>1: eof_layer=False ####
				
            if check: pdb.gimp_image_remove_layer(image, check)
                    
            if do_debug:
                old_handler = pdb.gimp_message_get_handler()	
                pdb.gimp_message_set_handler(MESSAGE_BOX)
                pdb.gimp_message(debug_info)
                pdb.gimp_message_set_handler(old_handler)         
	
        else:
            message_text = "Cannot use this file: It has no usable name."
            old_handler = pdb.gimp_message_get_handler()	
            pdb.gimp_message_set_handler(MESSAGE_BOX)
            pdb.gimp_message(message_text)
            pdb.gimp_message_set_handler(old_handler)    
		
	
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
	
######################################################################################################################################

    #### Useful Functions
# -------------------------------------------------------------------------------------
# -------------------------------------------------------------------------------------

def fn_test_for_custom_layer(img,l_id,l_name):
    # checks that the current layer is compatible with custom fonts
    test = False
    if len(l_name) == 1:
        test = True
    elif len(l_name) == 5 or len(l_name) == 6:
        
        if ".png" in l_name:
            test = True
            if len(l_name) == 6:
                part_name_val = unicode(l_name[0:2])
                if part_name_val <= 65535: test = False
    # select the contents of the layer
    pdb.gimp_image_select_item(img,2,l_id)
    # Test to see if the selection actually contains something
    sel_is_empty = pdb.gimp_selection_is_empty(img)
    # If there are no pixels to work with flag the layer as not viable to work on
    if sel_is_empty: test = False
    # Deselect
    pdb.gimp_selection_none(img)
    return test
	
######################################################################################################################################
	
def fn_selection_info(im):
    # returns selection info
    selection,x1,y1,x2,y2 = pdb.gimp_selection_bounds(im)
    s_width, s_height = x2 - x1, y2 - y1
	
    return selection, x1, x2, y1, y2, s_width, s_height


######################################################################################################################################
	
def fn_selection_amount(sel_value,r_ness,adjustment):
    # 
    value = (math.sqrt(sel_value)*r_ness)*adjustment
    return value

######################################################################################################################################
	
def proc_messagebox(message):
    old_handler = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2)
    pdb.gimp_message(str(message))
    pdb.gimp_message_set_handler(old_handler)
	
######################################################################################################################################
######################################################################################################################################
	
register(
    "python_fu_Custom_Font_File_Stitcher_v_03",
    "Provides options for stitching custom font files back together",
    "Provides options for stitching custom font files back together",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2024",
    "<Image>/Filters/Custom Fonts/Utility Tools/Custom Font File Stitcher v 0.3...",
    "RGB*",
    [
    (PF_TOGGLE, "stitch_option", "Re-join any children custom font files\nfound in the same folder as this parent image: ", 1),
    (PF_TOGGLE, "name_option", "Allow parent custom font files name\nending with number suffix: ", 0),
    ],
    [],
    do_stuff)

main()
